@@ -17,14 +17,9 @@ import java.io.FileWriter;
 import java.io.IOException;
 import java.lang.reflect.Field;
 import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
 import java.lang.reflect.Proxy;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.LinkedHashMap;
-import java.util.LinkedHashSet;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
+import java.util.*;
 import java.util.concurrent.Callable;
 
 /**
@@ -138,13 +133,20 @@ public class ReflectionConfigGenerator {
     }
 
     static final class Visitor {
-        Map<String, ReflectedClass> visited = new LinkedHashMap<String, ReflectedClass>();
+        Map<String, ReflectedClass> visited = new TreeMap<String, ReflectedClass>();
 
         Visitor() {
             getOrCreateClass(Method.class);
             getOrCreateClassName("java.lang.reflect.Executable").addMethod("getParameters");
             getOrCreateClassName("java.lang.reflect.Parameter").addMethod("getName");
 
+            // ANSI color enabled detection
+            getOrCreateClassName("java.lang.System").addMethod("console");
+            getOrCreateClassName("org.fusesource.jansi.AnsiConsole").addField("out", false);
+
+            // picocli 4.0
+            getOrCreateClassName("java.util.ResourceBundle").addMethod("getBaseBundleName");
+
             // type converters registered with reflection
             getOrCreateClassName("java.time.Duration").addMethod("parse", CharSequence.class);
             getOrCreateClassName("java.time.Instant").addMethod("parse", CharSequence.class);
@@ -212,21 +214,25 @@ public class ReflectionConfigGenerator {
             Field[] declaredFields = cls.getDeclaredFields();
             for (Field f : declaredFields) {
                 if (f.isAnnotationPresent(CommandLine.Spec.class)) {
-                    reflectedClass.addField(f.getName());
+                    reflectedClass.addField(f.getName(), isFinal(f));
                 }
                 if (f.isAnnotationPresent(CommandLine.ParentCommand.class)) {
-                    reflectedClass.addField(f.getName());
+                    reflectedClass.addField(f.getName(), isFinal(f));
                 }
                 if (f.isAnnotationPresent(CommandLine.Mixin.class)) {
-                    reflectedClass.addField(f.getName());
+                    reflectedClass.addField(f.getName(), isFinal(f));
                 }
                 if (f.isAnnotationPresent(CommandLine.Unmatched.class)) {
-                    reflectedClass.addField(f.getName());
+                    reflectedClass.addField(f.getName(), isFinal(f));
                 }
             }
             visitAnnotatedFields(cls.getSuperclass());
         }
 
+        private boolean isFinal(Field f) {
+            return (f.getModifiers() & Modifier.FINAL) == Modifier.FINAL;
+        }
+
         private void visitArgSpec(ArgSpec argSpec) throws NoSuchFieldException, IllegalAccessException {
             visitGetter(argSpec.getter());
             visitSetter(argSpec.setter());
@@ -280,8 +286,11 @@ public class ReflectionConfigGenerator {
 
         private void visitFieldBinding(Object fieldBinding) throws IllegalAccessException, NoSuchFieldException {
             Field field = (Field) accessibleField(fieldBinding.getClass(), REFLECTED_FIELD_BINDING_FIELD).get(fieldBinding);
+            getOrCreateClass(field.getDeclaringClass())
+                    .addField(field.getName(), isFinal(field));
+
             Object scope = accessibleField(fieldBinding.getClass(), REFLECTED_BINDING_FIELD_SCOPE).get(fieldBinding);
-            getOrCreateClass(scope.getClass()).addField(field.getName());
+            getOrCreateClass(scope.getClass());
         }
 
         private void visitMethodBinding(Object methodBinding) throws IllegalAccessException, NoSuchFieldException {
@@ -332,15 +341,15 @@ public class ReflectionConfigGenerator {
     }
     static class ReflectedClass {
         private final String name;
-        private final Set<ReflectedField> fields = new LinkedHashSet<ReflectedField>();
-        private final Set<ReflectedMethod> methods = new LinkedHashSet<ReflectedMethod>();
+        private final Set<ReflectedField> fields = new TreeSet<ReflectedField>();
+        private final Set<ReflectedMethod> methods = new TreeSet<ReflectedMethod>();
 
         ReflectedClass(String name) {
             this.name = name;
         }
 
-        ReflectedClass addField(String fieldName) {
-            fields.add(new ReflectedField(fieldName));
+        ReflectedClass addField(String fieldName, boolean isFinal) {
+            fields.add(new ReflectedField(fieldName, isFinal));
             return this;
         }
 
@@ -388,7 +397,7 @@ public class ReflectionConfigGenerator {
             return result;
         }
     }
-    static class ReflectedMethod {
+    static class ReflectedMethod implements Comparable<ReflectedMethod> {
         private final String name;
         private final String[] paramTypes;
 
@@ -418,12 +427,22 @@ public class ReflectionConfigGenerator {
             }
             return result.toString();
         }
-    }
-    static class ReflectedField {
-        private final String name;
 
-        ReflectedField(String name) {
+        public int compareTo(ReflectedMethod o) {
+            int result = name.compareTo(o.name);
+            if (result == 0) {
+                result = Arrays.toString(this.paramTypes).compareTo(Arrays.toString(o.paramTypes));
+            }
+            return result;
+        }
+    }
+    static class ReflectedField implements Comparable<ReflectedField> {
+        private final String name;
+        private final boolean isFinal;
+
+        ReflectedField(String name, boolean isFinal) {
             this.name = name;
+            this.isFinal = isFinal;
         }
         @Override public int hashCode() { return name.hashCode(); }
         @Override public boolean equals(Object o) {
@@ -432,7 +451,13 @@ public class ReflectionConfigGenerator {
 
         @Override
         public String toString() {
-            return String.format("{ \"name\" : \"%s\" }", name);
+            return isFinal
+                    ? String.format("{ \"name\" : \"%s\", \"allowWrite\" : true }", name)
+                    : String.format("{ \"name\" : \"%s\" }", name);
+        }
+
+        public int compareTo(ReflectedField o) {
+            return name.compareTo(o.name);
         }
     }
 }
