@@ -501,7 +501,7 @@ public class CommandLineCommandMethodTest {
             CommandLine.populateCommand(m, "-oout -r -vp1 p2".split(" "));
             fail("should fail: -v does not take an argument");
         } catch (UnmatchedArgumentException ex) {
-            assertEquals("Unknown option: -p1", ex.getMessage());
+            assertEquals("Unknown option: -p1 (while processing option: '-vp1')", ex.getMessage());
         }
     }
 
